# -*- coding: utf-8 -*-
# (c) 2012 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com

from django import http
from django.template import loader
from django.template.context import Context
from django.utils.log import getLogger
from django.views.decorators.csrf import requires_csrf_token
import uuid
from url_history.models import url_history

logger = getLogger('django.request')


@requires_csrf_token
def server_error(request, template_name='500.html', data=None):
    """
    500 error handler that logs the URL history and an error ID, and shows the
    error ID to the user so that a developer can find it in the logs later.

    Templates: `500.html`
    Context:
        error_id
            A unique identifier for the error that just happened.
    """

    error_id = str(uuid.uuid4())
    error_message = 'Error ID: %s' % error_id
    if hasattr(request, 'session'):
        error_message = '%s. URLs leading up to this error: %s' % (
            error_message, url_history(request.session))
    logger.error(error_message)

    t = loader.get_template(template_name)

    if data:
        data = dict(data)
    else:
        data = dict()
    data['error_id'] = error_id
    result = http.HttpResponseServerError(t.render(Context(data)))
    return result



# -*- coding: utf-8 -*-
# (c) 2012 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com

from django.conf.urls import patterns, url

urlpatterns = patterns('url_history.views',
    # Only for use by tests - doesn't add any useful functionality in production
    # (server_error is supposed to be used by assigning it to handler500)
    url(r'^test-server-error$', 'server_error', name='test-server-error'),
)



# -*- coding: utf-8 -*-
# (c) 2012 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com

"""
Tests for the history functionality that is used to provide a URL history with
bug reports.
"""

from django.core.urlresolvers import reverse
from url_history.models import url_history
from url_history.views import server_error
import django.test

class TestCase(django.test.TestCase):
    tags = ['service', ]


class HistoryTests(TestCase):

    def test_basic_history(self):
        urls = [
            '/first',
            '/something/second',
            '/something/third/'
            ]

        # visit the urls
        for url in urls:
            self.client.get(url)

        self.assertEqual(
            urls,
            url_history(self.client.session))

        # visit another url and check that the history changed
        new_url = '/four444th'
        self.client.get(new_url)
        urls.append(new_url)

        self.assertEqual(
            urls,
            url_history(self.client.session))

    def test_query_params_saved(self):
        url_with_params = '/assets/42/resize?width=200&height=300'
        self.assertContainsQueryString(url_with_params)

        self.client.get(url_with_params)

        self.assertEqual(
            [url_with_params],
            url_history(self.client.session)
        )

    def assertContainsQueryString(self, url):
        self.assertIn('?', url)

    def test_history_is_bounded(self):
        request_count = 50
        url = '/first'
        for i in xrange(request_count):
            self.client.get(url)
        self.assertLess(len(url_history(self.client.session)),
                        request_count)


class HistoryServerErrorViewTests(TestCase):
    """
    Tests for url_history.views.server_error.

    Depend on django-raise-exception-view being installed and it's urls.py
    being included in the urlconf.
    """

    def test_error_id_included_in_error_context(self):
        urls = [
            '/first11?foo=bar',
            '/something/second',
            '/tird/'
            ]

        # visit the urls
        for url in urls:
            self.client.get(url)

        response = self.client.get(reverse('test-server-error'))

        self.assertEqual(500, response.status_code)
        self.assertIsNotNone(response.context['error_id'])

    def test_error_ids_unique(self):
        error_id1 = self._call_server_error_view_and_return_error_id()
        error_id2 = self._call_server_error_view_and_return_error_id()
        self.assertNotEqual(error_id1, error_id2)

    def _call_server_error_view_and_return_error_id(self):
        response = self.client.get(reverse('test-server-error'))
        self.assertEqual(500, response.status_code)
        error_id = response.context['error_id']
        return error_id

    def test_handles_error_before_session_middleware(self):
        # request.session can be undefined if an error is hit on the user's
        # first request prior to the session middleware being reached
        request_factory = django.test.client.RequestFactory()
        request = request_factory.get(reverse('test-server-error'))
        try:
            error_id = server_error(request)
            self.assertIsNotNone(error_id)
        except AttributeError as e:
            self.fail('Should handle requests with no session gracefully: %s' % e)



# -*- coding: utf-8 -*-
# (c) 2012 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com

import collections


URL_HISTORY_KEY = 'url_history'
MAX_HISTORY_LENGTH = 20


def _get_url_history_queue(session):
    return collections.deque(
        iterable=_get_url_history_list(session),
        maxlen=MAX_HISTORY_LENGTH)


def _get_url_history_list(session):
    return session.get(URL_HISTORY_KEY,
                       [])


def _set_url_history_queue(request, history):
    request.session[URL_HISTORY_KEY] = list(history)


def url_history(session):
    """
    Return the last few URLs that were accessed in the given Django session.

    Only works if URLHistoryMiddleware is enabled.
    """

    return _get_url_history_list(session)



# -*- coding: utf-8 -*-
# (c) 2012 Bright Interactive Limited. All rights reserved.
# http://www.bright-interactive.com | info@bright-interactive.com

from url_history.models import  _get_url_history_queue, _set_url_history_queue


class URLHistoryMiddleware(object):
    """
    Stores the most recently visited URLs in the session.

    Requires sessions to be enabled.
    """

    def process_request(self, request):
        history = _get_url_history_queue(request.session)
        history.append(request.get_full_path())

        _set_url_history_queue(request, history)
